//***************************************************************************
//* address.cpp
//*
//* description    : abstraction of an IP address
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 24 2001
//* email          : cedric@delaunois.org
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#include "address.h"

Address::Address()
{
	buf[0] = 0;
	addr.sin_port = 0;
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = 0;
	bzero((void*)addr.sin_zero, sizeof(addr.sin_zero));
} 



Address::Address(const char* s) 
{
	buf[0] = 0;
	addr.sin_port = 0;
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = inet_addr(s);
	bzero((void*)addr.sin_zero, sizeof(addr.sin_zero));
}



Address::Address(const struct sockaddr* s) 
{
	buf[0] = 0;
	memcpy((void*)&addr, (void*)s, sizeof(addr));
}



Address::Address(const struct sockaddr_in s) 
{
	buf[0] = 0;
	memcpy((void*)&addr, (void*)&s, sizeof(addr));
}



Address::Address(const unsigned long s) 
{
	buf[0] = 0;
	addr.sin_port = 0;
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = s;
	bzero((void*)addr.sin_zero, sizeof(addr.sin_zero));
}



Address & Address::operator=(const Address &s) 
{
	if (this==&s) return *this;
	memcpy((void*)&addr, (void*)&s.addr, sizeof(addr));
	return *this;
}



int operator==(const Address &a1, const Address &a2) 
{
	return (a1.addr.sin_addr.s_addr==a2.addr.sin_addr.s_addr);
}



int operator!=(const Address &a1, const Address &a2) 
{
	return (a1.addr.sin_addr.s_addr!=a2.addr.sin_addr.s_addr);
}



int operator<(const Address &a1, const Address &a2) 
{
	return (a1.addr.sin_addr.s_addr<a2.addr.sin_addr.s_addr);
}



int operator>(const Address &a1, const Address &a2) 
{
	return (a1.addr.sin_addr.s_addr>a2.addr.sin_addr.s_addr);
}



int operator<=(const Address &a1, const Address &a2) 
{
	return (a1.addr.sin_addr.s_addr<=a2.addr.sin_addr.s_addr);
}



int operator>=(const Address &a1, const Address &a2) 
{
	return (a1.addr.sin_addr.s_addr>=a2.addr.sin_addr.s_addr);
}



ostream& operator << (ostream &out, const Address &a) {
	out << a.toChar();
	return out;
}


