//***************************************************************************
//* address.h
//*
//* description    : abstraction of an IP address
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 24 2001
//* email          : cedric@delaunois.org
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

class Address;

#ifndef _ADDRESS_H_
#define _ADDRESS_H_

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <iostream>
#include <string.h>

using namespace std;

class Address {

	friend int operator==(const Address &i1, const Address &i2);
	friend int operator!=(const Address &i1, const Address &i2);
	friend int operator<(const Address &i1, const Address &i2);
	friend int operator>(const Address &, const Address &);
	friend int operator<=(const Address &i1, const Address &i2);
	friend int operator>=(const Address &, const Address &);

	friend ostream& operator << (ostream &, const Address &);

 public:
	Address();
	Address(const char*);
	Address(const struct sockaddr*);
	Address(const struct sockaddr_in);
	Address(const unsigned long);
	Address & operator=(const Address &);

	inline unsigned long getAddr() const { return addr.sin_addr.s_addr; }
	inline unsigned short getPort() const { return addr.sin_port; }
	inline struct sockaddr* getSockaddr() const { return (struct sockaddr*)&addr; }
	inline char* toChar() const { return strncpy(buf, inet_ntoa(addr.sin_addr), 39); }
	inline bool isDefined() const { return addr.sin_addr.s_addr!=0; }

	inline void setAddr(unsigned long a) { addr.sin_addr.s_addr=a; }
	inline void setPort(unsigned short p) { addr.sin_port=p; }
	inline void undefine() { addr.sin_addr.s_addr=0; }

 private:
	struct sockaddr_in addr;
	mutable char buf[40];
};

#endif
