//***************************************************************************
//* arp.cpp
//*
//* description    : operations on ARP table
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 13 2001
//* email          : cedric@delaunois.org
//*
//* Some code is from iptunnel.c by Alexey Kuznetsov, <kuznet@ms2.inr.ac.ru>
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#include "arp.h"


/* Get the hardware address to a specified interface name */
int arp_getdevhw(char *ifname, struct sockaddr *sa)
{
    struct ifreq ifr;
    int fd;

    strcpy(ifr.ifr_name, ifname);

    if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	perror("socket");
	return(-1);
    }

    if (ioctl(fd, SIOCGIFHWADDR, &ifr) < 0) {
	fprintf(stderr, "arp: cant get HW-Address for `%s': %s.\n", ifname, strerror(errno));
	close(fd);
	return (-1);
    }

    close(fd);

    memcpy((char *) sa, (char *) &(ifr.ifr_hwaddr), sizeof(struct sockaddr));

    return 1;
}


/* Delete an entry from the ARP cache. */
int arp_del(char* addr, char* dev)
{
    struct arpreq req;
    struct sockaddr_in sin;
    int sockfd;

    if (!addr || !dev) return -1;

    memset((char *) &req, 0, sizeof(req));
    memset(&sin, 0, sizeof(struct sockaddr));
    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = inet_addr(addr);
 
    if (arp_getdevhw(dev, &req.arp_ha) < 0)
      return (-1);

    strncpy(req.arp_dev, dev, (sizeof req.arp_dev));

    memcpy((char *) &req.arp_pa, (char *) &sin, sizeof(struct sockaddr));

    req.arp_flags = ATF_PERM | ATF_PUBL;

    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	perror("socket");
	return(-1);
    }

    /* Call the kernel. */
    if (ioctl(sockfd, SIOCDARP, &req) < 0) {
        perror("SIOCDARP");
	close(sockfd);
	return (-1);
    }

    close(sockfd);
    return 1;
}


/* Set an entry in the ARP cache. */
int arp_set(char* addr, char* dev)
{
    struct arpreq req;
    struct sockaddr_in sin;
    int sockfd, flags;

    if (!addr || !dev) return -1;

    memset((char *) &req, 0, sizeof(req));
    memset(&sin, 0, sizeof(struct sockaddr));
    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = inet_addr(addr);
 
    if (arp_getdevhw(dev, &req.arp_ha) < 0)
      return (-1);

    strncpy(req.arp_dev, dev, (sizeof req.arp_dev));

    memcpy((char *) &req.arp_pa, (char *) &sin, sizeof(struct sockaddr));

    flags = ATF_PERM | ATF_COM | ATF_PUBL;
    req.arp_flags = flags;

    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	perror("socket");
	return(-1);
    }

    /* Call the kernel. */
    if (ioctl(sockfd, SIOCSARP, &req) < 0) {
        perror("SIOCSARP");
	close(sockfd);
	return (-1);
    }

    close(sockfd);
    return 1;
}

