//***************************************************************************
//* bind.cpp
//*
//* description    : Modelization of a RSIP bind
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 24 2001
//* email          : cedric@delaunois.org
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#include "bind.h"

Resources* Bind::pool = NULL;

Bind::Bind() 
{
	bindID   = 0;
	deadline = 0;
	resList  = NULL;
	tunnel   = NULL;
	nPorts   = 0;
	nIPs     = 0;
}


Bind::~Bind() 
{ 
	clearResources();
}


int Bind::addResource(Resource &r, bool aDC, bool pDC, 
		      unsigned char np, bool listen) 
{
	if (!pool) return -1;

	int result = pool->getResource(r, aDC, pDC, np, listen);

	if (result>0) {
		RE res;

		/* undefined address not allowed */
		if (!r.address.isDefined()) {
			pool->ungetResource(r, np);
			return -1;
		}

		res.r = r;
		res.range = np;
		if (!resList) resList = new deque<RE>();

		resList->push_back(res);

		if (np == 0) nIPs++;
		else nPorts+=np;
	}

	return result;
}


int Bind::addResource(Address& a, deque<uint16_t> &ports, 
		      bool aDC, bool listen)
{
	if (!pool) return -1;

	int result = pool->getResourceList(a, ports, aDC, listen);

	if (result>0) {

		if (!resList) resList = new deque<RE>();

		for (unsigned int i=0; i<ports.size(); i++) {
			RE res = { Resource(a, ports[i]), 1 };
			resList->push_back(res);
		}

		nPorts += ports.size();
	}

	return result;
}


void Bind::clearResources() 
{
	if (resList) {
		RE res;
		while (resList->size()) {
			res = resList->front();
			pool->ungetResource(res.r, res.range);
			resList->pop_front();
		}
		delete resList;
		resList = NULL;
	} 
	nPorts = 0;
	nIPs = 0;
}

