//**************************************************************************
//* common.h
//*
//* description    : Variables, Exceptions and common methods
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 11 2001
//* email          : cedric@delaunois.org
//**************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#ifndef _COMMON_H_
#define _COMMON_H_

#include <sys/types.h>
#include <netinet/in.h>
#include <unistd.h>
#include "version.h"
#include "config.h"
#include "address.h"

typedef struct _options Options;

// This structure contains the fields for all the parameters of the rsip server
struct _options
{
	Address ext_ip;                  // External IP
	Address int_ip;                  // Internal IP
	uint32_t initial_lease_time;          // Initial Lease Time of a resource
	uint32_t initial_register_lease_time; // Initial Register Lease Time for a client
	uint32_t max_lease_time;              // Max Lease Time of a resource
	uint32_t min_lease_time;              // Min Lease Time of a resource
	char* ext_if;                    // External Interface Name
	char* int_if;                    // Internal Interface Name
	char* cfg_file;                  // Name of config file
	char* status_file;               // Name of status file
	char* pid_file;                  // Name of pid file
	unsigned char refresh;           // Number of seconds between 2 check of the server IP
	unsigned short port;             // Port of the server
	unsigned short pktsize;          // Max size in bytes allowed for RSIP packets
	unsigned short rsa_method;       // Option for RSA-IP method
	unsigned short max_tcp_clients;  // Max # of clients using TCP
	unsigned short max_clients;      // Max # of clients
	unsigned short max_host_ports;   // Max # of ports reserved per client. 0->no RSAP-IP
	unsigned short rsap_low_port;
	unsigned short rsap_high_port;
	unsigned short listen_low_port;
	unsigned short listen_high_port;
	unsigned short net_delay;
	unsigned short mtu;
	int log_level;                   // Log level
	int output;                      // Output for the messages
	bool dump;                       // Dumps (1) or not (0) packets received
	bool daemon;                     // Runs in background (1) or not (0)
	bool stop;                       // Stopping (1) or not (0) the server
	bool arp;                        // Add an ARP entry for RSIP address ?
	bool force_iface;                // Force or not the iface to use by the tunnels
	bool check_ip;                   // Check for a change of the server IP
};

#include "exceptions.h"
#include "stdio.h"
#include "log.h"
#include "timer.h"
#include "shell.h"

// Global variables
extern Timer* tmr;         // The timer module
extern Shell* sh;          // To spawn external commands
extern Options* opt;       // Global options

#endif
