//***************************************************************************
//* ifconfig.cpp
//*
//* description    : basic configuration of tunnel interfaces
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 13 2001
//* email          : cedric@delaunois.org
//*
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#include "ifconfig.h"


/* Set a certain interface flag. */
static int set_flag(char *ifname, short flag)
{
	struct ifreq ifr;
	int skfd;

	skfd = socket(AF_INET, SOCK_DGRAM, 0);

	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
	if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) {
		fprintf(stderr, "%s: unknown interface: %s\n", 
			ifname,	strerror(errno));
		return -1;
	}
	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
	ifr.ifr_flags |= flag;
	if (ioctl(skfd, SIOCSIFFLAGS, &ifr) < 0) {
		perror("SIOCSIFFLAGS");
		return -1;
	}
	return 1;
}



int do_ifconfig(const char* name, const char* local, const char * remote, unsigned short mtu)
{
	int fd;
	struct ifreq ifr;
	struct sockaddr_in sinLoc, sinRem;

	if (!name || !local || !remote) return -1;

	strncpy(ifr.ifr_name, name, IFNAMSIZ);
    
	memset(&sinLoc, 0, sizeof(struct sockaddr));
	sinLoc.sin_family = AF_INET;
	sinLoc.sin_addr.s_addr = inet_addr(local);
    
	memset(&sinRem, 0, sizeof(struct sockaddr));
	sinRem.sin_family = AF_INET;
	sinRem.sin_addr.s_addr = inet_addr(remote);

	/* Create a channel to the NET kernel. */
	fd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fd < 0) return -2;

	memcpy((char *) &ifr.ifr_addr, (char *) &sinLoc,
	       sizeof(struct sockaddr));
	if (ioctl(fd, SIOCSIFADDR, &ifr) < 0) {
		fprintf(stderr, "ifconfig: SIOCSIFADDR: %s\n",
			strerror(errno));
		close(fd);
		return -3;

	}
    
	memcpy((char *) &ifr.ifr_dstaddr, (char *) &sinRem,
	       sizeof(struct sockaddr));

	if (set_flag(ifr.ifr_name, IFF_POINTOPOINT)<0 ||
	    set_flag(ifr.ifr_name, (IFF_UP | IFF_RUNNING))<0)
		return -4;

	if (ioctl(fd, SIOCSIFDSTADDR, &ifr) < 0) {
		fprintf(stderr, "ifconfig: SIOCSIFDSTADDR: %s\n",
			strerror(errno));
		close(fd);
		return -5;
	}

	if (mtu) {
		ifr.ifr_mtu = mtu;
		if (ioctl(fd, SIOCSIFMTU, &ifr) < 0) {
			fprintf(stderr, "SIOCSIFMTU: %s\n", strerror(errno));
			close(fd);
			return -6;
		}
	}
 
	close(fd);
	return 1;
}


