//***************************************************************************
//* interval.h
//*
//* description    : interval of integer
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 24 2001
//* email          : cedric@delaunois.org
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

class Interval;

#ifndef _INTERVAL_H_
#define _INTERVAL_H_

#include <iostream>

using namespace std;

class Interval {

	friend int operator==(const Interval &i1, const Interval &i2);
	// Returns true if one interval is included in the other
	// Ex: [3,5] == [2,10] returns true


	friend int operator!=(const Interval &i1, const Interval &i2);
	// Returns true if i1 and i2 and disjoined
	// Ex: [3,5] != [2,10] returns false


	friend int operator<(const Interval &i1, const Interval &i2);
	// Returns true if i1.ub < i2.lb
	// Ex: [3,5] < [6, 10] returns true


	friend int operator>(const Interval &, const Interval &);
	// Returns true if i1.lb > i2.ub
	// Ex: [6,10] > [1, 3] returns true


	friend ostream& operator << (ostream &, const Interval &);

 public:
	Interval(int l=0, int u=0) : lb(l), ub(u) {}
	Interval(const Interval& i) : lb(i.lb), ub(i.ub) {}
	Interval & operator=(const Interval &);
	
	int lb, ub;
};

#endif
