//***************************************************************************
//* netutils.cpp
//*
//* description    : Network utilities
//*
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 24 2001
//* email          : cedric@delaunois.org
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#include "netutils.h"


int checkModule(char* tnl)
{
 	struct ip_tunnel_parm p;
	struct ifreq ifr;
	int fd;

	if (tnl==NULL) return -1;
	
	bzero(&ifr, sizeof(ifr));
	memset(&p, 0, sizeof(p));

	strncpy(p.name, tnl, IFNAMSIZ);
	strncpy(ifr.ifr_name, tnl, IFNAMSIZ);
	ifr.ifr_ifru.ifru_data = (char*)&p;
	
	fd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fd < 0) return -2;
	
	if (ioctl(fd, SIOCGETTUNNEL, &ifr)<0) {
		close(fd);
		return -3;
	}

	close(fd);

	return 1;
}



int getIP(char* ifname, Address &ip) 
{
	struct ifreq ifr;
	int fd;
	
	if (!ifname) return -1;

	bzero(&ifr, sizeof(ifr));

	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
 	
	// Create a channel to the NET kernel.
	fd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fd < 0) return -2;
	
	if (ioctl(fd, SIOCGIFADDR, &ifr)<0) {
		close(fd);
		return -3;
	}

	close(fd);

	ip.setAddr(((struct sockaddr_in*)&ifr.ifr_addr)->sin_addr.s_addr);

	return 1;
}



int checkAddress(char* address) 
{
	int a[5];
	int n;
	for (n=0; n<5; n++) a[n]=-1;
	n = sscanf(address, "%d.%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3], &a[4]);
	if (n!=4) return 0; 
	else {
		if (a[0]==0) return 0;
		for (n=0; n<4; n++) {
			if ((a[n]<0) || (a[n]>255)) return 0;
		} 
		return 1; 
	}
}



void dumpHex(char const *buf, int len) 
{
	int i;
	FILE* debugFile = stderr;

	for (i=0; i<len; i++) {
		if (i == len-1 || !((i+1)%16)) {
			fprintf(debugFile, "0x%.2x\n", (unsigned char) buf[i]);
		} else if (!((i+1)%8)) {
			fprintf(debugFile, "0x%.2x-", (unsigned char) buf[i]);
		} else if (!((i+1)%2)) {
			fprintf(debugFile, "0x%.2x ", (unsigned char) buf[i]);
		} else {
			fprintf(debugFile, "0x%.2x ", (unsigned char) buf[i]);
		}
	}
}


