//**************************************************************************
//* rsipd.h 
//*
//* description    : rsipd.h 
//*                  Genaral declarations
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 11 2001
//* email          : cedric@delaunois.org
//* compilation flags : -lnsl -lsocket -lpthread
//**************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#ifndef _RSIPD_H_
#define _RSIPD_H_

#include <pthread.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/stat.h>
#include <string>

#include "common.h"
#include "server.h"
#include "proto.h"
#include "bind.h"
#include "protocol.h"
#include "rsipClient.h"

#ifdef _HAVE_SLP
#include "slp.h"
#endif

#ifndef VERSION
#define VERSION            "Unknown"
#endif

#define RET_SUCCESS        0
#define RET_USAGE          1
#define RET_VERSION        2
#define ERR_BAD_OPTION    -1
#define ERR_CONFIG        -3
#define ERR_EXT_IP        -4
#define ERR_INT_IP        -5
#define ERR_EXT_IF        -6
#define ERR_INT_IF        -7
#define ERR_RANGE         -8
#define ERR_BAD_RIGHTS    -9
#define ERR_IPIP          -10
#define ERR_IPGRE         -11
#define ERR_MODPROBE      -12

// Global variables
Timer*   tmr;        // A global timer module
Shell*   sh;         // To spawn external commands
Options* opt;        // Global options

//****************************************************************************
//* Function sigTerm
//* SPEC: process signals received
//* PRE : src is the signal received
//****************************************************************************
void sigTerm(int src);

//****************************************************************************
//* Function usage. 
//* PRE : name of the program
//* POST: shows usage of the program
//****************************************************************************
void usage(char const *name);

//****************************************************************************
//* Function isRoot
//* SPEC: check that the program is running as root
//* PRE : / 
//* POST: returns 1 if root
//*               0 otherwise
//****************************************************************************
int isRoot();

//****************************************************************************
//* Function monitor
//* SPEC: Monitor the IP of the server
//* PRE : server started
//* POST: pool updated if IP has changed
//*       returns 1 if succes, 0 if an error occured
//****************************************************************************
int monitor(Options* o, Resources* pool);

//****************************************************************************
//*
//* Function daemonStart
//*
//* Detach a daemon procss from login session context
//*
//****************************************************************************
void daemonStart();

//****************************************************************************
//*
//* Function readOptions
//*
//* Initialize options defined in configuration file of name "name"
//*
//* Returns  1 if no error
//*         -1 otherwise
//****************************************************************************
int readOptions(Options* o, const char *name);

//****************************************************************************
//* Function init
//* SPEC: Initialize the options
//* PRE : /
//* POST: - options initialized
//*       - Returns >=1 if success
//*                 the error code <0 otherwise
//****************************************************************************
int init(Options* &o, int argc, char** argv);

//****************************************************************************
//* SPEC: Initialize the pool of Resources
//* PRE : o is initialized
//* POST: - The pool is filled with resources from config file
//*       - opt->max_clients is set to the number of IP +5 if it is equal to 0
//*       - The resources in file descriptors are increased accordingly
//*       - Returns false if an error occured
//*               true otherwise
//****************************************************************************
bool init(Resources* &r, Options* o);

//****************************************************************************
//* Function clean
//* SPEC: properly terminate the program. Call at exit().
//****************************************************************************
void clean(void);

//****************************************************************************
//* Function dumpState
//* SPEC: dump the current state of the server
//****************************************************************************
void dumpState(int src);

//****************************************************************************
//* Function writepid
//* SPEC: write the pid of the current process in the file
//****************************************************************************
void writepid(const char* file);

#endif
