//***************************************************************************
//* server.cpp
//*
//* description    : Class Server : implementation
//*                  Waiting and connecting new clients.
//*
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 13 2001
//* email          : cedric@delaunois.org
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#include "server.h"



Server::Server(Options* o) 
{
	opts = o;
	lstr = NULL;
}



Server::~Server() 
{
	if (lstr) delete lstr;
	lstr = NULL;
}



int Server::start(proto protocol) 
{
	if (protocol>3 || protocol<1) protocol=TCP; 

	if (opts->port<1024) {
		logmsg(LOG_WARNING, "Invalid port specified. Using default: %d\n", DEFAULT_RSIP_PORT);
		opts->port = DEFAULT_RSIP_PORT;
	}


	// Starts the listener
	lstr = new Listener(opts);


	// Opens the UDP end point connection
	if (protocol && UDP) {
		UDPEndPoint* uep = new UDPEndPoint();

		try {
			uep->openPort(opts->int_if, opts->port, NULL);
		} catch (NetworkException) {
			logconsole(LOG_ERR, "Can't open socket. Maybe another server is running ?\n");
			delete uep;
			return 0;
		}

		lstr->listen(uep);
	}


	// Opens the TCP end point listener
	if (protocol && TCP) {
		TCPEndPoint* tep = new TCPEndPoint();

		try {
			tep->openPort(opts->int_if, opts->port, NULL);
		} catch (NetworkException) {
			logconsole(LOG_ERR, "Can't open socket. Maybe another server is running ?\n");
			delete tep;
			return 0;
		}

		lstr->accept(tep);
	}

	// NOTE: uep and tep are deleted when listener is stopped

	lstr->start();

	return 1;
}

