//***************************************************************************
//* server.h
//*
//* description    : Main server
//*                  Waiting and connecting new clients.
//*
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 13 2001
//* email          : cedric@delaunois.org
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

#ifndef _SERVER_H_
#define _SERVER_H_

#include <unistd.h>
#include <pthread.h>
#include <signal.h>
#include "common.h"
#include "tcpEndPoint.h"
#include "udpEndPoint.h"
#include "netutils.h"
#include "listener.h"
#include "proto.h"

#define DEFAULT_RSIP_PORT 4555

class Server {
 public:

	Server(Options* o = NULL);
	// Constructor, with options
	// POST : server stopped


	~Server();
	// Destructor, waiting for the termination of all the clients
	// POST : server stopped


	int start(proto protocol);
	// Starts the server
	// Treatement of clients connecting on port 'port' with the specified protocol
	// PRE : server stopped
	//       - if options are defined :
	//           o->port : valid port (between 1025 and 65534)
	//       - protocol : logic or with TCP or UDP
	// POST: if success, return 1 and the serveur is running, waiting for connections on that port
	//       otherwise return 0


	inline void stop() { if (lstr) delete lstr; lstr=NULL; }
	// Stops & waits for server to end
	// PRE : server running
	// POST: server stopped


	inline void setOption(Options* o) { opts = o; }
	// Defines the options
	// PRE : /
	// POST: the options for the server are o

 private:

	Options* opts;        // Options
	Listener* lstr;       // The listener

};


#endif
