//***************************************************************************
//* shell.h
//*
//* description    : class which spawn a shell command
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Dec 21 2001
//* email          : cedric@delaunois.org
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

class Shell;

#ifndef _SHELL_H
#define _SHELL_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <pthread.h>
#include <deque>

#include "common.h"

class Shell {
 public:

	Shell(bool dump=false);
	// Constructor
	// PRE : must NOT be constructed within a thread

	~Shell();
	// Destructor

	int start();
	// Start the server
	// PRE : /
	// POST: returns 0 if an error occured
	//               1 otherwise

	int execute(const char *path, const char*arg, ...);
	// Execute the command line with the specified args
	// PRE : path is the name of the program to execute
	//       arg are the arguments to pass to the program
	//       The last argument MUST BE NULL
	// POST: return the return code of the program
	//              or -1 if an error occured (can't fork, etc.)

	int execute(const char *path, deque<const char*> arg);
	// Execute the command line with the specified args
	// PRE : path is the name of the program to execute
	//       arg is the list of the arguments to pass to the program
	// POST: return the return code of the program
	//              or -1 if an error occured (can't fork, etc.)

 private:

	void serve();
	bool started;
	bool dump;
	pid_t server;
	FILE* channel[2];       // For pipe for command
	FILE* result[2];        // For pipe for results
	pthread_mutex_t mutex;

};

#endif
