//***************************************************************************
//* timer.h
//*
//* description    : class Timer
//*                  Schedule various tasks
//*
//* copyright      : (C) 2001 Cedric de Launois
//*                  Universite Catholique de Louvain, Belgium
//* begin          : Sept 24 2001
//* email          : cedric@delaunois.org
//***************************************************************************

/****************************************************************************
 * Copyright (c) 1998 Free Software Foundation, Inc.                        *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

class Timer;

#ifndef _TIMER_H_
#define _TIMER_H_

#include <pthread.h>
#include <time.h>
#include <unistd.h>
#include <deque>
#include <map>
#include "common.h"

#define DFLT_PERIOD 500000  // in usec, cannot be greater than 999999

class Timer {

 public:
	enum State { RUNNING, STOPPING, STOPPED };

	Timer();
	// Constructor
	// Create and start the timer
	// PRE : /
	// POST: the timer is started


	~Timer();
	// Destructor
	// Stop and destroy the timer
	// The remaining tasks are not processed


	int schedule(void* (*function)(void*), void* arg, unsigned int sec);
	// Tell the timer to call function(arg) in sec seconds in the future
	// PRE :
	// POST: returns 0 if the timer is not RUNNING
	//       returns 1 otherwise

 private:

	typedef struct _task {
		void* (*function)(void*);
		void* arg;
	} Task;

	unsigned int number;           // Number of tasks being scheduled
	State status;                  // End Point Handler State
	pthread_t timer;               // Thread listener
	pthread_mutex_t mutex;         // To protect the table
	static unsigned long period;   // Reschedule after 'period' usec

	map<time_t, deque<Task>*> tasks;
	// The tables<deadline, function to call with arg>

	void doSchedule();

	friend void runTimer(Timer*);   // The timer thread
};

void runTimer(Timer* t);
#endif
